/*************************************************************
** gallerysubmit.c                                          **
**                                                          **
**    Developed by Seung-young Kim <nobreak@kr.FreeBSD.org> **
**                           Last updated at April 17, 1999 **
**                                                          **
**                                Korea FreeBSD Users Group **
*************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <qDecoder.h>

#define DB		"gallery.db"
#define DBREQUEST	"gallery.request.db"

#define HEADER		"galleryheader.html"
#define TAILER		"gallerytailer.html"

#define TRUE		(1)
#define FALSE		(0)

#define SENDMAIL	"/usr/sbin/sendmail -t -i"

struct query {
  char *category;
  char *organization;
  char *description;
  char *url;
  char *contact;
};

int isrecordexist(struct query *query, char *filename, char record[][256]);
int replacechar(char *str, char char1, char char2);
int checkurl(char *url);
void sendmail(struct query *query);

int main(void) {
  FILE *fp;
  struct query query;
  struct tm *today;
  char *returl;
  char datestr[8+1];
  char record[6][256];

  if((returl = qValue("returl")) == NULL) qError("Query(returl) not found!");

  /* Query Fetch */
#define queryFetch(NAME) { if ((query.NAME = qRemoveSpace(qValue(#NAME))) == NULL) qError("Please fill in the field %s.", #NAME); if (!strcmp(query.NAME, "")) qError("Please fill in the field %s", #NAME); else replacechar(query.NAME, '|', ':'); }
  queryFetch(category);
  queryFetch(organization);
  queryFetch(description);
  queryFetch(url);
  queryFetch(contact);
#undef queryFetch
  if(checkurl(query.url) == FALSE) qError("Invalid URL %s", query.url);
  if(qCheckEmail(query.contact) == FALSE) qError("Invalid contact address %s", query.contact);

  /* Adjust illegal characters */
  replacechar(query.organization, 0x22, 0x27); /* 0x22=" , 0x28=' */
  replacechar(query.organization, '<', '{');
  replacechar(query.organization, '>', '}');
  replacechar(query.description, '<', '{');
  replacechar(query.description, '>', '}');

  /* Checking exist records from db*/
  if(isrecordexist(&query, DBREQUEST, record) == TRUE) qError("Already submitted in %s: %s (%s) at %s", record[0], record[1], record[2], record[5]);
  if(isrecordexist(&query, DB, record)        == TRUE) qError("Already registered in %s: %s (%s) at %s", record[0], record[1], record[2], record[5]);

  /* Amke time string */
  today = qGetTime();
  sprintf(datestr, "%04d%02d%02d", today->tm_year, today->tm_mon, today->tm_mday);

  /* Store */
  if((fp = fopen(DBREQUEST, "at")) == NULL) qError("Fatal: please report this error to administrator.");
  fprintf(fp, "%s|%s|%s|%s|%s|%s\n", query.category, query.organization, query.url, query.contact, query.description, datestr);
  fclose(fp);

  /* stdout */
  qContentType("text/html");
  qSendFile(HEADER);

  printf("<h2 align=\"left\"><font color=\"#660000\">FreeBSD-KR   ּż մϴ.</font></h2>\n");
  printf("<br clear=\"all\">\n");
  printf("<p>Ǿϴ.\n");
  printf("ó ϷǾ Ǳ  24-48 ð ҿ˴ϴ.\n");
  printf("մϴ.\n");

  printf("<p><h2>%s</h2>\n", query.category);
  printf("<blockquote><table>\n");
  printf("<tr>\n");
  printf("  <td align=\"right\"><b>Organization:</b></td>\n");
  printf("  <td>%s</td>\n", query.organization);
  printf("</tr>\n");
  printf("<tr>\n");
  printf("  <td align=\"right\"><b>Description:</b></td>\n");
  printf("  <td>%s</td>\n", query.description);
  printf("</tr>\n");
  printf("<tr>\n");
  printf("  <td align=\"right\"><b>URL:</td>\n");
  printf("  <td>%s</td>\n", query.url);
  printf("<tr>\n");
  printf("  <td align=\"right\"><b>Contact:</b></td>\n");
  printf("  <td>%s</td>\n", query.contact);
  printf("</tr>\n");
  printf("</table></blockquote>\n");

  printf("<p><a href=\"%s\">Return to the Gallery Home...</a>\n", returl);

  qSendFile(TAILER);

  sendmail(&query);

  qFree();
  return 0;  
}

int isrecordexist(struct query *query, char *filename, char record[][256]) {
  int status = FALSE;
  
  if(qAwkOpen(filename, '|') == FALSE) return status;
  for(status = FALSE; qAwkNext(record) != -1;) {
    if(!strcmp(query->organization, record[1]) || !strcmp(query->url, record[2])) {
      status = TRUE;
      break;
    }
  }
  qAwkClose();
  
  return status;
}

int replacechar(char *str, char char1, char char2) {
  int count;
  
  for(count = 0; *str != '\0'; str++) if (*str == char1) { *str = char2; count++; }
  
  return count;
}

int checkurl(char *url) {
  int colon = 0;
  int slash = 0;
  int dot = 0;
  
  if (strncmp(url, "http://", strlen("http://")) && strncmp(url, "ftp://", strlen("ftp://"))) return FALSE;
  if(url[strlen(url) - 1] == '/') url[strlen(url) - 1] = '\0';

  for(; *url; url++) {
    if((*url >= '0') && (*url <= '9')) continue;
    else if((*url >= 'A') && (*url <= 'Z')) continue;
    else if((*url >= 'a') && (*url <= 'z')) continue;
    else if(*url == '-') continue;

    switch(*url) {
      case ':': { colon++; continue; }
      case '/': { slash++; continue; }
      case '.': { dot++; continue; }
      default : { return FALSE; }
    }
  }

  if(colon != 1 || slash != 2 || dot == 0) return FALSE;
  
  return TRUE;
}

void sendmail(struct query *query) {
  FILE *fp;

  if ((fp = popen(SENDMAIL, "w")) == NULL) qError("Fatal: Please report this problem to administrator.");
  fprintf(fp, "From: \"FreeBSD-KR Gallery Project\" <gallery@kr.FreeBSD.org>\n");
  fprintf(fp, "Reply-To: \"FreeBSD-KR Gallery Project\" <gallery@kr.FreeBSD.org>\n");
  fprintf(fp, "To: \"%s\" <%s>\n", query->organization, query->contact);
  fprintf(fp, "Bcc: \"Gallery Administrator\" <gallery@kr.FreeBSD.org>\n");
  fprintf(fp, "Subject: FreeBSD-KR   ּż մϴ.\n");
  fprintf(fp, "Organization: Korea FreeBSD Users Group\n");
  fprintf(fp, "\n");

  fprintf(fp, "%s ȳϽʴϱ. Korea FreeBSD Users Group Դϴ.\n", query->organization);
  fprintf(fp, "\n");
  fprintf(fp, "FreeBSD-KR ַ  Ͽ ּż  մϴ.\n");
  fprintf(fp, "   û   Ǿ ˸ ڵ\n");
  fprintf(fp, "Դϴ. ش û  ó ϷǾ ǹ\n");
  fprintf(fp, "ʽϴ.\n");
  fprintf(fp, "\n");
  fprintf(fp, " o Organization...: %s\n", query->organization);
  fprintf(fp, " o Descriptions...: %s\n", query->description);
  fprintf(fp, " o URL............: %s\n", query->url);
  fprintf(fp, " o Category.......: %s\n", query->category);
  fprintf(fp, " o Contact........: %s\n", query->contact);
  fprintf(fp, "\n");
  fprintf(fp, "ó ϷǾ ڷᰡ Ǳ  24-48 ð\n");
  fprintf(fp, "ҿǸ,  Ϸ  ٽ ˸  ߼ ˴ϴ.\n");
  fprintf(fp, "http://www.kr.FreeBSD.org/gallery/\n");
  fprintf(fp, "\n");
  fprintf(fp, "츮     ڹ üμ  \n");
  fprintf(fp, " FreeBSD  մϴ.  ôٸ,\n");
  fprintf(fp, "FreeBSD-KR ְϴ پ Ʈ  ֽʽÿ. \n");
  fprintf(fp, "http://www.kr.FreeBSD.org/projects/\n");
  fprintf(fp, "\n");
  fprintf(fp, "``ÿ  ״ Ʒ    ִ°,\n");
  fprintf(fp, "                  ɾ ̴.''\n");
  fprintf(fp, "\n");
  fprintf(fp, "մϴ.\n");
  fprintf(fp, "- FreeBSD-KR Gallery Project <gallery@kr.FreeBSD.org>\n");

  pclose(fp);
}
